#
# device.py
# Python bindings for libparted (built on top of the _ped Python module).
#
# Copyright (C) 2009-2013 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.
#
# Author(s): David Cantrell <dcantrell@redhat.com>
#            Alex Skinner <alex@lx.lc>
#

import sys
if sys.version_info >= (3, ):
    long = int

import math
import warnings

import parted
import _ped

from parted.decorators import localeC

class Device(object):
    """Device()

       Device represents a phyiscal piece of hardware in the system, e.g. a
       disk.  A Device should be considered a low-level and operating system
       specific interface to the hardware.

       A number of read-only properties about the Device are available.

       For information on the individual methods, see help(Device.METHODNAME)"""

    @localeC
    def __init__(self, path=None, PedDevice=None):
        """Create a new Device object based on the specified path or the
           already existing _ped.Device object.  You must provide either a
           path (e.g., "/dev/sda") or an existing _ped.Device object, but
           not both."""

        if PedDevice:
            self.__device = PedDevice
        elif path is not None:
            self.__device = _ped.device_get(path)
        else:
            raise parted.DeviceException("no path or PedDevice specified")

    def __eq__(self, other):
        return not self.__ne__(other)

    def __ne__(self, other):
        if not isinstance(self, other.__class__):
            return True

        return self.model != other.model or self.path != other.path or self.type != other.type or self.length != other.length

    def __getCHS(self, geometry):
        return (geometry.cylinders, geometry.heads, geometry.sectors)

    @property
    def model(self):
        """Model name and vendor of this device."""
        return self.__device.model

    @property
    def path(self):
        """Filesystem node path of this device (e.g., /dev/sda)."""
        return self.__device.path

    @property
    def type(self):
        """Type of this device.  An integer constant corresponding
           to one of the parted.DEVICE_* values.
        """
        return self.__device.type

    @property
    def sectorSize(self):
        """Sector size (in bytes) for this device."""
        return self.__device.sector_size

    @property
    def physicalSectorSize(self):
        """Physical sector size (in bytes) for this device.  Not always
           the same as sectorSize, but is a multiple of sectorSize.
        """
        return self.__device.phys_sector_size

    @property
    def length(self):
        """The size of this device in sectors."""
        return self.__device.length

    @property
    def openCount(self):
        """How many times the open() method has been called on this device."""
        return self.__device.open_count

    @property
    def readOnly(self):
        """True if the device is currently in read-only mode, False
           otherwise.
        """
        return bool(self.__device.read_only)

    @property
    def externalMode(self):
        """True if external access mode is currently activated on this
           device, False otherwise.  External access mode has to be used
           if you want to use an external command on the device while
           you are currently using it in pyparted.
        """
        return bool(self.__device.external_mode)

    @property
    def dirty(self):
        """True if the device is marked dirty, False otherwise."""
        return bool(self.__device.dirty)

    @property
    def bootDirty(self):
        """True if the device is marked boot dirty, False otherwise."""
        return bool(self.__device.boot_dirty)

    @property
    def host(self):
        """The host value of this device."""
        return self.__device.host

    @property
    def did(self):
        """The did value of this device."""
        return self.__device.did

    @property
    def busy(self):
        """True if this device is busy, False otherwise."""
        return self.__device.is_busy()

    @property
    def hardwareGeometry(self):
        """A 3-tuple representing the hardware geometry of this device.
           The tuple is in order of cylinders, heads, and sectors.
        """
        return self.__getCHS(self.__device.hw_geom)

    @property
    def biosGeometry(self):
        """A 3-tuple representing the BIOS geometry of this device.
           The tuple is in order of cylinders, heads, and sectors.
        """
        return self.__getCHS(self.__device.bios_geom)

    def __str__(self):
        s = ("parted.Device instance --\n"
             "  model: %(model)s  path: %(path)s  type: %(type)s\n"
             "  sectorSize: %(sectorSize)s  physicalSectorSize:  %(physSectorSize)s\n"
             "  length: %(length)s  openCount: %(openCount)s  readOnly: %(readOnly)s\n"
             "  externalMode: %(external)s  dirty: %(dirty)s  bootDirty: %(bootDirty)s\n"
             "  host: %(host)s  did: %(did)s  busy: %(busy)s\n"
             "  hardwareGeometry: %(hardwareGeom)s  biosGeometry: %(biosGeom)s\n"
             "  PedDevice: %(ped)r" %
             {"model": self.model, "path": self.path, "type": self.type,
              "sectorSize": self.sectorSize, "physSectorSize": self.physicalSectorSize,
              "length": self.length, "openCount": self.openCount, "readOnly": self.readOnly,
              "external": self.externalMode, "dirty": self.dirty, "bootDirty": self.bootDirty,
              "host": self.host, "did": self.did, "busy": self.busy,
              "hardwareGeom": self.hardwareGeometry, "biosGeom": self.biosGeometry,
              "ped": self.__device})
        return s

    @localeC
    def clobber(self):
        """Remove all identifying signatures of the partition table."""
        return self.__device.clobber()

    @localeC
    def open(self):
        """Open this Device for read operations."""

        return self.__device.open()

    @localeC
    def close(self):
        """Close this Device.  Used after open() method calls."""

        return self.__device.close()

    @localeC
    def destroy(self):
        """Destroy this Device.  Operating system specific."""

        return self.__device.destroy()

    @localeC
    def removeFromCache(self):
        """Remove this Device from the internal libparted device cache."""

        return self.__device.cache_remove()

    @localeC
    def beginExternalAccess(self):
        """Set up the Device for use by an external program.  Call this method
           before running an external program that uses the Device."""

        return self.__device.begin_external_access()

    @localeC
    def endExternalAccess(self):
        """Turn off external access mode for the Device.  Call this method once
           your external program has finished using the Device."""

        return self.__device.end_external_access()

    @localeC
    def read(self, start, count):
        """From the sector indentified by start, read and return count sectors
           from the Device."""

        return self.__device.read(start, count)

    @localeC
    def write(self, buf, start, count):
        """From the sector identified by start, write count sectors from
           buffer to the Device."""

        return self.__device.write(buf, start, count)

    @localeC
    def sync(self, fast=False):
        """Perform a operating-system specific sync(2) operation on the
           Device.  If fast is True, try to perform a fast sync(2)."""

        if fast:
            return self.__device.sync_fast()
        else:
            return self.__device.sync()

    @localeC
    def check(self, start, count):
        """From the sector identified by start, perform an operating
           system specific check on count sectors."""
        return self.__device.check(start, count)

    @localeC
    def startSectorToCylinder(self, sector):
        """Return the closest cylinder (round down) to sector on
           this Device."""
        (_, heads, sectors) = self.biosGeometry
        return long(math.floor((float(sector) / (heads * sectors)) + 1))

    @localeC
    def endSectorToCylinder(self, sector):
        """Return the closest cylinder (round up) to sector on
           this Device."""
        (_, heads, sectors) = self.biosGeometry
        return long(math.ceil(float((sector + 1)) / (heads * sectors)))

    @localeC
    def startCylinderToSector(self, cylinder):
        """Return the sector corresponding to cylinder as a
           starting cylinder on this Device."""
        (_, heads, sectors) = self.biosGeometry
        return long((cylinder - 1) * (heads * sectors))

    @localeC
    def endCylinderToSector(self, cylinder):
        """Return the sector corresponding to cylinder as an
           ending cylinder on this Device."""
        (_, heads, sectors) = self.biosGeometry
        return long(((cylinder) * (heads * sectors)) - 1)

    def getSize(self, unit="MB"):
        """Return the size of the Device in the unit specified.  The unit
           is given as a string corresponding to one of the following
           abbreviations:  b (bytes), KB (kilobytes), MB (megabytes), GB
           (gigabytes), TB (terabytes).  An invalid unit string will raise a
           SyntaxError exception.  The default unit is MB."""
        warnings.warn("use the getLength method", DeprecationWarning)
        lunit = unit.lower()

        if lunit not in parted._exponent.keys():
            raise SyntaxError("invalid unit %s given" % (unit))

        size = float(self.__device.length)
        size /= math.pow(1024.0, parted._exponent[lunit])
        size *= self.sectorSize

        return size

    @localeC
    def getLength(self, unit='sectors'):
        """Return the length of the device in sectors. Optionally, a SI or
           IEC prefix followed by a 'B' may be given in order to convert the
           length into bytes. The allowed values include B, kB, MB, GB, TB, KiB,
           MiB, GiB, and TiB."""
        sectors = self.__device.length
        if unit == "sectors":
            return sectors
        return parted.formatBytes(sectors * self.sectorSize, unit)

    @localeC
    def getConstraint(self):
        """Return a Constraint defining the limitations imposed by
           this Device."""
        return parted.Constraint(PedConstraint=self.__device.get_constraint())

    @property
    @localeC
    def minimalAlignedConstraint(self):
        """Return a Constraint defining the limitations and minimal advisable
           alignment imposed by this Device."""
        constraint = self.__device.get_minimal_aligned_constraint()
        return parted.Constraint(PedConstraint=constraint)

    @property
    @localeC
    def optimalAlignedConstraint(self):
        """Return a Constraint defining the limitations and optimal
           alignment imposed by this Device."""
        constraint = self.__device.get_optimal_aligned_constraint()
        return parted.Constraint(PedConstraint=constraint)

    @property
    @localeC
    def minimumAlignment(self):
        """Return an Alignment defining the minimum alignment for this Device."""
        alignment = self.__device.get_minimum_alignment()
        return parted.Alignment(PedAlignment=alignment)

    @property
    @localeC
    def optimumAlignment(self):
        """Return an Alignment defining the optimum alignment for this Device."""
        alignment = self.__device.get_optimum_alignment()
        return parted.Alignment(PedAlignment=alignment)

    def getPedDevice(self):
        """Return the _ped.Device object contained in this Device.
           For internal module use only."""
        return self.__device



#
# Test cases for the methods in the _ped module itself - just the pyunit
# and pynatmath files.
#
# Copyright (C) 2008-2014  Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.
#
# Red Hat Author(s): Chris Lumens <clumens@redhat.com>
#                    David Cantrell <dcantrell@redhat.com>
#

import os
import _ped
import unittest

from tests.baseclass import BuildList, RequiresDevice, RequiresFileSystem

# One class per method, multiple tests per class.  For these simple methods,
# that seems like good organization.  More complicated methods may require
# multiple classes and their own test suite.
class PartitionFlagGetNameTestCase(unittest.TestCase):
    def runTest(self):
        for f in ['PARTITION_BOOT', 'PARTITION_ROOT', 'PARTITION_SWAP',
                  'PARTITION_HIDDEN', 'PARTITION_RAID', 'PARTITION_LVM',
                  'PARTITION_LBA', 'PARTITION_HPSERVICE',
                  'PARTITION_PALO', 'PARTITION_PREP',
                  'PARTITION_MSFT_RESERVED',
                  'PARTITION_APPLE_TV_RECOVERY',
                  'PARTITION_BIOS_GRUB', 'PARTITION_DIAG',
                  'PARTITION_MSFT_DATA', 'PARTITION_IRST',
                  'PARTITION_ESP']:
            if not hasattr(_ped, f):
                continue
            attr = getattr(_ped, f)
            self.assertNotEqual(_ped.partition_flag_get_name(attr), "", "Could not get name for flag _ped.%s" % f)

        self.assertRaises(ValueError, _ped.partition_flag_get_name, -1)
        self.assertRaises(ValueError, _ped.partition_flag_get_name, 1000)

class PartitionFlagGetByNameTestCase(unittest.TestCase):
    def runTest(self):
        for f in ["boot", "root", "swap", "hidden", "raid", "lvm", "lba",
                  "hp-service", "palo", "prep", "msftres", "bios_grub",
                  "msftdata", "irst", "esp"]:
            self.assertNotEqual(_ped.partition_flag_get_by_name(f), "", "Could not get flag %s" % f)

        self.assertEqual(_ped.partition_flag_get_by_name("nosuchflag"), 0)

class PartitionFlagNextTestCase(unittest.TestCase):
    def runTest(self):
        # We should get TypeError when the parameter is invalid
        self.assertRaises(TypeError, _ped.partition_flag_next)
        self.assertRaises(TypeError, _ped.partition_flag_next, 'blah')

        # First flag is 0, keep getting flags until we loop back around
        # to zero.  Make sure each flag we get is an integer.
        flag = _ped.partition_flag_next(0)
        self.assertEqual(type(flag).__name__, 'int')

        while True:
            flag = _ped.partition_flag_next(flag)
            if not flag:
                break
            self.assertEqual(type(flag).__name__, 'int')

class DiskFlagGetNameTestCase(unittest.TestCase):
    def runTest(self):
        for f in [_ped.DISK_CYLINDER_ALIGNMENT]:
            self.assertNotEqual(_ped.disk_flag_get_name(f), "", "Could not get name for flag %s" % f)

        self.assertRaises(ValueError, _ped.disk_flag_get_name, -1)
        self.assertRaises(ValueError, _ped.disk_flag_get_name, 1000)

class DiskFlagGetByNameTestCase(unittest.TestCase):
    def runTest(self):
        for f in ["cylinder_alignment"]:
            self.assertNotEqual(_ped.disk_flag_get_by_name(f), 0, "Could not get flag %s" % f)

        self.assertEqual(_ped.disk_flag_get_by_name("nosuchflag"), 0)

class DiskFlagNextTestCase(unittest.TestCase):
    def runTest(self):
        # We should get TypeError when the parameter is invalid
        self.assertRaises(TypeError, _ped.disk_flag_next)
        self.assertRaises(TypeError, _ped.disk_flag_next, 'blah')

        # First flag is 0, keep getting flags until we loop back around
        # to zero.  Make sure each flag we get is an integer.
        flag = _ped.disk_flag_next(0)
        self.assertEqual(type(flag).__name__, 'int')

        while True:
            flag = _ped.disk_flag_next(flag)
            if not flag:
                break
            self.assertEqual(type(flag).__name__, 'int')

class ConstraintNewFromMinMaxTestCase(RequiresDevice):
    def runTest(self):
        self.assertRaises(TypeError, _ped.constraint_new_from_min_max, None)

        # min is required to be within max, so test various combinations of
        # that not being the case.
        self.assertRaises(_ped.CreateException, _ped.constraint_new_from_min_max,
                          _ped.Geometry(self._device, 0, 10),
                          _ped.Geometry(self._device, 15, 25))
        self.assertRaises(_ped.CreateException, _ped.constraint_new_from_min_max,
                          _ped.Geometry(self._device, 10, 20),
                          _ped.Geometry(self._device, 15, 25))

        # Now test a correct call.
        minimum = _ped.Geometry(self._device, 10, 20)
        maximum = _ped.Geometry(self._device, 0, 30)
        constraint = _ped.constraint_new_from_min_max(minimum, maximum)

        self.assertIsInstance(constraint, _ped.Constraint)
        self.assertTrue(constraint.is_solution(_ped.Geometry(self._device, 10, 20)))
        self.assertFalse(constraint.is_solution(_ped.Geometry(self._device, 11, 20)))
        self.assertTrue(constraint.is_solution(_ped.Geometry(self._device, 5, 25)))
        self.assertTrue(constraint.is_solution(_ped.Geometry(self._device, 0, 30)))
        self.assertFalse(constraint.is_solution(_ped.Geometry(self._device, 0, 35)))

class ConstraintNewFromMinTestCase(RequiresDevice):
    def runTest(self):
        self.assertRaises(TypeError, _ped.constraint_new_from_min, None)

        minimum = _ped.Geometry(self._device, 10, 20)
        constraint = _ped.constraint_new_from_min(minimum)

        self.assertIsInstance(constraint, _ped.Constraint)
        self.assertTrue(constraint.is_solution(_ped.Geometry(self._device, 10, 20)))
        self.assertTrue(constraint.is_solution(_ped.Geometry(self._device, 5, 25)))
        self.assertFalse(constraint.is_solution(_ped.Geometry(self._device, 11, 19)))
        self.assertFalse(constraint.is_solution(_ped.Geometry(self._device, 15, 25)))

class ConstraintNewFromMaxTestCase(RequiresDevice):
    def runTest(self):
        self.assertRaises(TypeError, _ped.constraint_new_from_max, None)

        maximum = _ped.Geometry(self._device, 10, 20)
        constraint = _ped.constraint_new_from_max(maximum)

        self.assertIsInstance(constraint, _ped.Constraint)
        self.assertTrue(constraint.is_solution(_ped.Geometry(self._device, 10, 20)))
        self.assertFalse(constraint.is_solution(_ped.Geometry(self._device, 5, 25)))
        self.assertTrue(constraint.is_solution(_ped.Geometry(self._device, 11, 19)))
        self.assertFalse(constraint.is_solution(_ped.Geometry(self._device, 15, 25)))

class ConstraintAnyTestCase(RequiresDevice):
    def runTest(self):
        self.assertRaises(TypeError, _ped.constraint_any, None)

        constraint = _ped.constraint_any(self._device)
        self.assertIsInstance(constraint, _ped.Constraint)

        for testGeom in [_ped.Geometry(self._device, 0, 5),
                         _ped.Geometry(self._device, 10, 25),
                         _ped.Geometry(self._device, 0, 100)]:
            self.assertTrue(constraint.is_solution(testGeom))

class ConstraintExactTestCase(RequiresDevice):
    def runTest(self):
        geom = _ped.Geometry(self._device, 0, 100)

        self.assertRaises(TypeError, _ped.constraint_exact, None)

        constraint = _ped.constraint_exact(geom)
        self.assertIsInstance(constraint, _ped.Constraint)

        for testGeom in [_ped.Geometry(self._device, 1, 100),
                         _ped.Geometry(self._device, 0, 99),
                         _ped.Geometry(self._device, 10, 20),
                         _ped.Geometry(self._device, 50, 101)]:
            self.assertFalse(constraint.is_solution(testGeom))

        self.assertTrue(constraint.is_solution(_ped.Geometry(self._device, 0, 100)))

class DeviceGetTestCase(RequiresDevice):
    def runTest(self):
        # Try getting the device we just made.
        self.assertIsInstance(_ped.device_get(self.path), _ped.Device)

        # Try getting a device that doesn't exist.
        self.assertRaises(_ped.IOException, _ped.device_get, "/blah/whatever")
        self.assertRaises(_ped.IOException, _ped.device_get, "")
        self.assertRaises(_ped.DeviceException, _ped.device_get, None)

@unittest.skipUnless(os.geteuid() == 0, 'Requires root')
class DeviceGetNextTestCase(unittest.TestCase, BuildList):
    def runTest(self):
        # Make sure there are some devices in the system first and then
        # make a list out of them.  That's easier to work with.
        _ped.device_probe_all()
        lst = self.getDeviceList(_ped.device_get_next)

        # Now the test cases.
        self.assertGreater(len(lst), 0)
        self.assertRaises(TypeError, _ped.device_get_next, None)

        for ele in lst:
            self.assertIsInstance(ele, _ped.Device)

        self.assertRaises(IndexError, _ped.device_get_next, lst[-1])

class DeviceProbeAllTestCase(RequiresDevice, BuildList):
    def runTest(self):
        # Since we inherit from RequiresDevice, we can test that the temp
        # device we created is in the results list.  I can't really think of
        # any other way to test this method except by getting a list of devices
        # via some other mechanism and comparing that to the device_probe_all
        # results.
        _ped.device_probe_all()
        lst = self.getDeviceList(_ped.device_get_next)

        self.assertGreater(len(lst), 0)
        self.assertGreater(
            len([e for e in lst if e.path.startswith("/tmp/temp-device-")]), 0)

class DeviceFreeAllTestCase(RequiresDevice):
    def runTest(self):
        _ped.device_probe_all()
        self.assertEqual(_ped.device_free_all(), None)

class DiskTypeGetTestCase(unittest.TestCase):
    def runTest(self):
        for d in ["aix", "amiga", "bsd", "dvh", "gpt", "loop", "mac", "msdos",
                  "pc98","sun"]:
            t = _ped.disk_type_get(d)
            self.assertIsInstance(t, _ped.DiskType)
            self.assertEqual(t.name, d)

        self.assertRaises(_ped.UnknownTypeException, _ped.disk_type_get, "nosuch")

class DiskTypeGetNextTestCase(unittest.TestCase, BuildList):
    def runTest(self):
        lst = self.getDeviceList(_ped.disk_type_get_next)
        self.assertGreater(len(lst), 0)
        self.assertRaises(TypeError, _ped.device_get_next, None)

        for ele in lst:
            self.assertIsInstance(ele, _ped.DiskType)

        self.assertRaises(IndexError, _ped.disk_type_get_next, lst[-1])

class FileSystemProbeTestCase(RequiresFileSystem):
    def runTest(self):
        ty = _ped.file_system_probe(self._geometry)

        for name in self._fileSystemType.keys():
            if name == 'ext2':
                self.assertEqual(ty.name, name)
            else:
                self.assertNotEqual(ty.name, name)

class FileSystemProbeSpecificTestCase(RequiresFileSystem):
    def runTest(self):
        for (name, ty,) in self._fileSystemType.items():
            if name == 'ext2':
                result = _ped.file_system_probe_specific(ty, self._geometry)

                # XXX: this should work
                # we're getting
                #     ValueError: object comparing to must be a _ped.Geometry
                # at runtime.  works fine in pdb.
                #self.assertEqual(result, self._geometry)

                self.assertIsInstance(result, _ped.Geometry)
                self.assertEqual(result.start, self._geometry.start)
                self.assertLessEqual(result.end, self._geometry.end)
                self.assertLessEqual(result.length, self._geometry.length)
                self.assertEqual(result.dev, self._device)
            else:
                result = _ped.file_system_probe_specific(ty, self._geometry)
                self.assertEqual(result, None)

class FileSystemTypeGetTestCase(unittest.TestCase):
    def runTest(self):
        for f in ["affs0", "amufs", "apfs1", "asfs", "btrfs", "ext2", "ext3", "ext4", "fat16",
                  "fat32", "hfs", "hfs+", "hfsx", "hp-ufs", "jfs", "linux-swap",
                  "ntfs", "reiserfs", "sun-ufs", "xfs"]:
            # may be missing some filesystem types depending on the parted
            # build on the test system
            try:
                t = _ped.file_system_type_get(f)
                self.assertIsInstance(t, _ped.FileSystemType, "Could not get fs type %s" % t)
            except _ped.UnknownTypeException:
                pass

        self.assertRaises(_ped.UnknownTypeException, _ped.file_system_type_get, "nosuch")

class FileSystemTypeGetNextTestCase(unittest.TestCase, BuildList):
    def runTest(self):
        lst = self.getDeviceList(_ped.file_system_type_get_next)
        self.assertGreater(len(lst), 0)
        self.assertRaises(TypeError, _ped.file_system_type_get_next, None)

        for ele in lst:
            self.assertIsInstance(ele, _ped.FileSystemType)

        self.assertRaises(IndexError, _ped.file_system_type_get_next, lst[-1])

class PartitionTypeGetNameTestCase(unittest.TestCase):
    def runTest(self):
        for t in [_ped.PARTITION_METADATA, _ped.PARTITION_FREESPACE,
                  _ped.PARTITION_EXTENDED, _ped.PARTITION_LOGICAL]:
            self.assertNotEqual(_ped.partition_type_get_name(t), "", "Could not get name for flag %s" % t)

class UnitSetDefaultTestCase(unittest.TestCase):
    def setUp(self):
        self._initialDefault = _ped.unit_get_default()

    def tearDown(self):
        _ped.unit_set_default(self._initialDefault)

    def runTest(self):
        for v in [_ped.UNIT_BYTE, _ped.UNIT_CHS, _ped.UNIT_COMPACT,
                  _ped.UNIT_CYLINDER, _ped.UNIT_GIBIBYTE, _ped.UNIT_GIGABYTE,
                  _ped.UNIT_KIBIBYTE, _ped.UNIT_KILOBYTE, _ped.UNIT_MEBIBYTE,
                  _ped.UNIT_MEGABYTE, _ped.UNIT_PERCENT, _ped.UNIT_SECTOR,
                  _ped.UNIT_TEBIBYTE, _ped.UNIT_TERABYTE]:
            _ped.unit_set_default(v)
            self.assertEqual(_ped.unit_get_default(), v, "Could not set unit default to %s" % v)

        self.assertRaises(ValueError, _ped.unit_set_default, -1)
        self.assertRaises(ValueError, _ped.unit_set_default, 1000)

class UnitGetDefaultTestCase(unittest.TestCase):
    def runTest(self):
        self.assertGreaterEqual(_ped.unit_get_default(), 0)

class UnitGetSizeTestCase(RequiresDevice):
    def runTest(self):
        self.assertEqual(self._device.unit_get_size(_ped.UNIT_SECTOR), 512)
        self.assertEqual(self._device.unit_get_size(_ped.UNIT_BYTE), 1)
        self.assertEqual(self._device.unit_get_size(_ped.UNIT_KILOBYTE), 1000)
        self.assertEqual(self._device.unit_get_size(_ped.UNIT_MEGABYTE), 1000000)
        self.assertEqual(self._device.unit_get_size(_ped.UNIT_GIGABYTE), 1000000000)
        self.assertEqual(self._device.unit_get_size(_ped.UNIT_TERABYTE), 1000000000000)
        self.assertEqual(self._device.unit_get_size(_ped.UNIT_KIBIBYTE), 1024)
        self.assertEqual(self._device.unit_get_size(_ped.UNIT_MEBIBYTE), 1048576)
        self.assertEqual(self._device.unit_get_size(_ped.UNIT_GIBIBYTE), 1073741824)
        self.assertEqual(self._device.unit_get_size(_ped.UNIT_TEBIBYTE), 1099511627776)
        self.assertEqual(self._device.unit_get_size(_ped.UNIT_CYLINDER), 65536)
        self.assertEqual(self._device.unit_get_size(_ped.UNIT_CHS), 512)
        self.assertEqual(self._device.unit_get_size(_ped.UNIT_PERCENT), self._device.length * self._device.sector_size // 100)
        self.assertRaises(ValueError, self._device.unit_get_size, _ped.UNIT_COMPACT)

class UnitGetNameTestCase(unittest.TestCase):
    def runTest(self):
        self.assertEqual(_ped.unit_get_name(_ped.UNIT_BYTE), 'B')
        self.assertEqual(_ped.unit_get_name(_ped.UNIT_CHS), 'chs')
        self.assertEqual(_ped.unit_get_name(_ped.UNIT_COMPACT), 'compact')
        self.assertEqual(_ped.unit_get_name(_ped.UNIT_CYLINDER), 'cyl')
        self.assertEqual(_ped.unit_get_name(_ped.UNIT_GIBIBYTE), 'GiB')
        self.assertEqual(_ped.unit_get_name(_ped.UNIT_GIGABYTE), 'GB')
        self.assertEqual(_ped.unit_get_name(_ped.UNIT_KIBIBYTE), 'kiB')
        self.assertEqual(_ped.unit_get_name(_ped.UNIT_KILOBYTE), 'kB')
        self.assertEqual(_ped.unit_get_name(_ped.UNIT_MEBIBYTE), 'MiB')
        self.assertEqual(_ped.unit_get_name(_ped.UNIT_MEGABYTE), 'MB')
        self.assertEqual(_ped.unit_get_name(_ped.UNIT_PERCENT), '%')
        self.assertEqual(_ped.unit_get_name(_ped.UNIT_SECTOR), 's')
        self.assertEqual(_ped.unit_get_name(_ped.UNIT_TEBIBYTE), 'TiB')
        self.assertEqual(_ped.unit_get_name(_ped.UNIT_TERABYTE), 'TB')

        self.assertRaises(ValueError, _ped.unit_get_name, -1)
        self.assertRaises(ValueError, _ped.unit_get_name, 1000)

class UnitGetByNameTestCase(unittest.TestCase):
    def runTest(self):
        self.assertEqual(_ped.unit_get_by_name('B'), _ped.UNIT_BYTE)
        self.assertEqual(_ped.unit_get_by_name('chs'), _ped.UNIT_CHS)
        self.assertEqual(_ped.unit_get_by_name('compact'), _ped.UNIT_COMPACT)
        self.assertEqual(_ped.unit_get_by_name('cyl'), _ped.UNIT_CYLINDER)
        self.assertEqual(_ped.unit_get_by_name('GiB'), _ped.UNIT_GIBIBYTE)
        self.assertEqual(_ped.unit_get_by_name('GB'), _ped.UNIT_GIGABYTE)
        self.assertEqual(_ped.unit_get_by_name('kiB'), _ped.UNIT_KIBIBYTE)
        self.assertEqual(_ped.unit_get_by_name('kB'), _ped.UNIT_KILOBYTE)
        self.assertEqual(_ped.unit_get_by_name('MiB'), _ped.UNIT_MEBIBYTE)
        self.assertEqual(_ped.unit_get_by_name('MB'), _ped.UNIT_MEGABYTE)
        self.assertEqual(_ped.unit_get_by_name('%'), _ped.UNIT_PERCENT)
        self.assertEqual(_ped.unit_get_by_name('s'), _ped.UNIT_SECTOR)
        self.assertEqual(_ped.unit_get_by_name('TiB'), _ped.UNIT_TEBIBYTE)
        self.assertEqual(_ped.unit_get_by_name('TB'), _ped.UNIT_TERABYTE)

        self.assertRaises(_ped.UnknownTypeException, _ped.unit_get_by_name, "blargle")



# setup.py script for pyparted
# Copyright (C) 2011-2013 Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# Author(s): David Cantrell <dcantrell@redhat.com>
#            Alex Skinner <alex@lx.lc>
#
# pylint: skip-file

import subprocess
import glob
import os
import platform
import sys
from distutils.ccompiler import new_compiler
from distutils.errors import CompileError
from distutils.errors import LinkError
from distutils.core import setup
from distutils.core import Extension

pyparted_version = '3.10.7'
python_version = sys.version_info

need_libparted_version = '2.3'
need_python_version = (2, 7)

if python_version < need_python_version:
    raise RuntimeError("pyparted requires Python version %d.%d or higher"
                       % need_python_version)
# Recipe from:
# http://code.activestate.com/recipes/502261-python-distutils-pkg-config/
def pkgconfig(*packages, **kwargs):
    flag_map = {'-I': 'include_dirs', '-L': 'library_dirs', '-l': 'libraries'}
    for token in subprocess.check_output(["pkg-config", "--libs", "--cflags"] + list(packages)).decode('utf-8').split():

        kwargs.setdefault(flag_map.get(token[:2]), []).append(token[2:])
    return kwargs

def check_mod_version(module, version):
    modversion = subprocess.check_output(["pkg-config", "--modversion", module])
    if not float(modversion) >= float(version):
        sys.stderr.write("*** Minimum required %s version: %s, found: %s\n" % (module, version, modversion,))
        sys.exit(1)
    return

check_mod_version('libparted', need_libparted_version)

# This list is in the format necessary for the define_macros parameter
# for an Extension() module definition.  See:
# http://docs.python.org/distutils/setupscript.html#preprocessor-options
features = [('PYPARTED_VERSION', "\"%s\"" % pyparted_version)]

setup(name='pyparted',
      version=pyparted_version,
      author='pyparted Development Team',
      author_email='pyparted-devel@redhat.com',
      url='https://fedorahosted.org/pyparted/',
      description='Python bindings for GNU parted',
      license='GPLv2+',
      packages=['parted'],
      package_dir={'parted': 'src/parted'},
      ext_modules=[Extension('_ped',
                             glob.glob(os.path.join('src', '*.c')),
                             define_macros=features,
                             **pkgconfig('libparted',
                                         include_dirs=['include']))
                  ])



#
# Python bindings for libparted (built on top of the _ped Python module).
#
# Copyright (C) 2009 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.
#
# Red Hat Author(s): Peter Jones <pjones@redhat.com>
#

import locale
import functools

def localeC(fn):
    # setlocale is not thread-safe, and anaconda (at least) may call this from
    # another thread.  This is just a luxury to have untranslated tracebacks,
    # so it's not worth tracebacking itself.
    def _setlocale(l):
        try:
            locale.setlocale(locale.LC_MESSAGES, l)
        # pylint: disable=bare-except
        except:
            pass

    @functools.wraps(fn)
    def new(*args, **kwds):
        oldlocale = locale.getlocale(locale.LC_MESSAGES)
        _setlocale('C')
        try:
            ret = fn(*args, **kwds)
        finally:
            _setlocale(oldlocale)
        return ret
    return new



#
# Copyright (C) 2008-2011  Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.
#
# Red Hat Author(s): Chris Lumens <clumens@redhat.com>
#                    David Cantrell <dcantrell@redhat.com>

import _ped
import parted
import os
import tempfile
import unittest

# Base class for any test case that requires a temp device node
class RequiresDeviceNode(unittest.TestCase):
    def setUp(self):
        self.addCleanup(self.removeTempDevice)

        (fd, self.path) = tempfile.mkstemp(prefix="temp-device-")
        f = os.fdopen(fd)
        f.seek(140000)
        os.write(fd, b"0")

    def removeTempDevice(self):
        if self.path and os.path.exists(self.path):
            os.unlink(self.path)

# Base class for any test case that requires a _ped.Device or parted.Device
# object first.
class RequiresDevice(RequiresDeviceNode):
    def setUp(self):
        RequiresDeviceNode.setUp(self)
        self._device = _ped.device_get(self.path)
        self.device = parted.getDevice(self.path)

# Base class for any test case that requires a filesystem on a device.
class RequiresFileSystem(unittest.TestCase):
    def setUp(self):
        self.addCleanup(self.removeTempDevice)

        self._fileSystemType = {}
        ty = _ped.file_system_type_get_next()
        self._fileSystemType[ty.name] = ty

        while True:
            try:
                ty = _ped.file_system_type_get_next(ty)
                self._fileSystemType[ty.name] = ty
            except (IndexError, TypeError, _ped.UnknownTypeException):
                break

        (fd, self.path,) = tempfile.mkstemp(prefix="temp-device-")
        f = os.fdopen(fd)
        f.seek(140000)
        os.write(fd, b"0")
        f.close()

        os.system("mke2fs -F -q %s" % (self.path,))

        self._device = _ped.device_get(self.path)
        self._geometry = _ped.Geometry(self._device, 0, self._device.length - 1)

    def removeTempDevice(self):
        if self.path and os.path.exists(self.path):
            os.unlink(self.path)

# Base class for certain alignment tests that require a _ped.Device
class RequiresDeviceAlignment(RequiresDevice):
    def setUp(self):
        RequiresDevice.setUp(self)

    def roundDownTo(self, sector, grain_size):
        if sector < 0:
            shift = sector % grain_size + grain_size
        else:
            shift = sector % grain_size

        return sector - shift

    def roundUpTo(self, sector, grain_size):
        if sector % grain_size:
            return self.roundDownTo(sector, grain_size) + grain_size
        else:
            return sector

    def closestInsideGeometry(self, alignment, geometry, sector):
        if alignment.grain_size == 0:
            if alignment.is_aligned(geometry, sector) and \
               ((geometry is None) or geometry.test_sector_inside(sector)):
                return sector
            else:
                return -1

        if sector < geometry.start:
            sector += self.roundUpTo(geometry.start - sector,
                                     alignment.grain_size)

        if sector > geometry.end:
            sector -= self.roundUpTo(sector - geometry.end,
                                     alignment.grain_size)

        if not geometry.test_sector_inside(sector):
            return -1

        return sector

    def closest(self, sector, a, b):
        if a == -1:
            return b

        if b == -1:
            return a

        if abs(sector - a) < abs(sector - b):
            return a
        else:
            return b

# Base class for any test case that requires a labeled device
class RequiresLabeledDevice(RequiresDevice):
    def setUp(self):
        RequiresDevice.setUp(self)
        os.system("parted -s %s mklabel msdos" % (self.path,))

# Base class for any test case that requires a _ped.Disk or parted.Disk.
class RequiresDisk(RequiresDevice):
    def setUp(self):
        RequiresDevice.setUp(self)
        self._disk = _ped.disk_new_fresh(self._device, _ped.disk_type_get("msdos"))
        self.disk = parted.Disk(PedDisk=self._disk)

# Base class for any test case that requires a filesystem made and mounted.
class RequiresMount(RequiresDevice):
    def setUp(self):
        self.addCleanup(self.removeMountpoint)
        RequiresDevice.setUp(self)
        self.mountpoint = None

    def mkfs(self):
        os.system("mkfs.ext2 -F -q %s" % self.path)

    def doMount(self):
        self.mountpoint = tempfile.mkdtemp()
        os.system("mount -o loop %s %s" % (self.path, self.mountpoint))

    def removeMountpoint(self):
        if self.mountpoint and os.path.exists(self.mountpoint):
            os.system("umount %s" % self.mountpoint)
            os.rmdir(self.mountpoint)

# Base class for any test case that requires a _ped.Partition.
class RequiresPartition(RequiresDisk):
    def setUp(self):
        RequiresDisk.setUp(self)
        self._part = _ped.Partition(disk=self._disk, type=_ped.PARTITION_NORMAL,
                                    start=0, end=100, fs_type=_ped.file_system_type_get("ext2"))

# Base class for any test case that requires a hash table of all
# _ped.DiskType objects available
class RequiresDiskTypes(unittest.TestCase):
    def setUp(self):
        self.disktype = {}
        ty = _ped.disk_type_get_next()
        self.disktype[ty.name] = ty

        while True:
            try:
                ty = _ped.disk_type_get_next(ty)
                self.disktype[ty.name] = ty
            except (IndexError, TypeError, _ped.UnknownTypeException):
                break

# Base class for any test case that requires a list being built via successive
# calls of some function.  The function must raise IndexError when there's no
# more output to add to the return list.  This class is most useful for all
# those _get_next methods.
class BuildList:
    def getDeviceList(self, func):
        lst = []
        prev = None

        while True:
            try:
                if not prev:
                    prev = func()
                else:
                    prev = func(prev)

                lst.append(prev)
            except IndexError:
                break

        return lst



#
# Copyright (C) 2009-2011  Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.
#
# Red Hat Author(s): Chris Lumens <clumens@redhat.com>
#                    David Cantrell <dcantrell@redhat.com>
#

import _ped

from tests.baseclass import RequiresDevice

# One class per method, multiple tests per class.  For these simple methods,
# that seems like good organization.  More complicated methods may require
# multiple classes and their own test suite.
class ConstraintNewTestCase(RequiresDevice):
    def runTest(self):
        align1 = _ped.Alignment(10, 5)
        align2 = _ped.Alignment(10, 5)
        geom1 = _ped.Geometry(self._device, 0, 50)
        geom2 = _ped.Geometry(self._device, 25, 50)

        # Check that not passing enough args to _ped.Constraint.__init__ is caught.
        self.assertRaises(TypeError, _ped.Constraint)
        self.assertRaises(TypeError, _ped.Constraint, align1, align2)

        # Or the parameters in the wrong order.
        self.assertRaises(TypeError, _ped.Constraint, align1, align2, 10, 100,
                          geom1, geom2)

        # And then the correct way of creating a _ped.Constraint.
        c = _ped.Constraint(align1, align2, geom1, geom2, 10, 100)
        self.assertIsInstance(c, _ped.Constraint)

class ConstraintGetSetTestCase(RequiresDevice):
    def setUp(self):
        RequiresDevice.setUp(self)
        align1 = _ped.Alignment(10, 5)
        align2 = _ped.Alignment(10, 5)
        geom1 = _ped.Geometry(self._device, 0, 50)
        geom2 = _ped.Geometry(self._device, 25, 50)

        self.c = _ped.Constraint(align1, align2, geom1, geom2, min_size=10,
                                 max_size=100)

    def runTest(self):
        # Test that passing the kwargs to __init__ works.
        self.assertEqual(self.c.min_size, 10)
        self.assertEqual(self.c.max_size, 100)
        self.assertIsInstance(self.c.start_align, _ped.Alignment)
        self.assertIsInstance(self.c.end_align, _ped.Alignment)
        self.assertIsInstance(self.c.start_range, _ped.Geometry)
        self.assertIsInstance(self.c.end_range, _ped.Geometry)

        # Test that setting directly and getting with getattr works.
        self.c.min_size = 15
        self.c.max_size = 75

        self.assertEqual(getattr(self.c, "min_size"), 15)
        self.assertEqual(getattr(self.c, "max_size"), 75)
        self.assertIsInstance(getattr(self.c, "start_align"), _ped.Alignment)
        self.assertIsInstance(getattr(self.c, "end_align"), _ped.Alignment)
        self.assertIsInstance(getattr(self.c, "start_range"), _ped.Geometry)
        self.assertIsInstance(getattr(self.c, "end_range"), _ped.Geometry)

        # Test that setting with setattr and getting directly works.
        setattr(self.c, "min_size", 10)
        setattr(self.c, "max_size", 90)

        self.assertEqual(self.c.min_size, 10)
        self.assertEqual(self.c.max_size, 90)

        # Test that values have the right type.
        self.assertRaises(TypeError, setattr, self.c, "min_size", "string")

        # Test that looking for invalid attributes fails properly.
        self.assertRaises(AttributeError, getattr, self.c, "blah")

        # We really shouldn't be allowed to overwrite objects stored in a
        # _ped.Constraint, but for now there's no way to prevent it.
        self.c.end_range = 47
        self.assertEqual(self.c.end_range, 47)

class ConstraintDuplicateTestCase(RequiresDevice):
    def setUp(self):
        RequiresDevice.setUp(self)
        align1 = _ped.Alignment(10, 0)
        align2 = _ped.Alignment(10, 0)
        geom1 = _ped.Geometry(self._device, 0, 50)
        geom2 = _ped.Geometry(self._device, 25, 50)

        self.c = _ped.Constraint(align1, align2, geom1, geom2, min_size=10,
                                 max_size=100)
        self.dup = self.c.duplicate()

    def runTest(self):
        self.assertEqual(self.c.min_size, self.dup.min_size)
        self.assertEqual(self.c.max_size, self.dup.max_size)

        # duplicate methods should do a deepcopy, so self.dup should have
        # different references, but the same contents.
        self.assertNotEqual(repr(self.c), repr(self.dup))

        self.assertNotEqual(repr(self.c.start_align), repr(self.dup.start_align))
        self.assertEqual(self.c.start_align.offset, self.dup.start_align.offset)
        self.assertEqual(self.c.start_align.grain_size, self.dup.start_align.grain_size)

        self.assertNotEqual(repr(self.c.end_align), repr(self.dup.end_align))
        self.assertEqual(self.c.end_align.offset, self.dup.end_align.offset)
        self.assertEqual(self.c.end_align.grain_size, self.dup.end_align.grain_size)

        self.assertNotEqual(repr(self.c.start_range), repr(self.dup.start_range))
        self.assertNotEqual(repr(self.c.start_range.dev), repr(self.dup.start_range.dev))
        self.assertEqual(self.c.start_range.dev.path, self.dup.start_range.dev.path)
        self.assertEqual(self.c.start_range.start, self.dup.start_range.start)
        self.assertEqual(self.c.start_range.length, self.dup.start_range.length)
        self.assertEqual(self.c.start_range.end, self.dup.start_range.end)

        self.assertNotEqual(repr(self.c.end_range), repr(self.dup.end_range))
        self.assertNotEqual(repr(self.c.end_range.dev), repr(self.dup.end_range.dev))
        self.assertEqual(self.c.end_range.dev.path, self.dup.end_range.dev.path)
        self.assertEqual(self.c.end_range.start, self.dup.end_range.start)
        self.assertEqual(self.c.end_range.length, self.dup.end_range.length)
        self.assertEqual(self.c.end_range.end, self.dup.end_range.end)

class ConstraintIntersectTestCase(RequiresDevice):
    def setUp(self):
        RequiresDevice.setUp(self)
        align1 = _ped.Alignment(10, 0)
        align2 = _ped.Alignment(10, 0)
        geom1 = _ped.Geometry(self._device, 0, 50)
        geom2 = _ped.Geometry(self._device, 25, 50)

        self.c1 = _ped.Constraint(align1, align2, geom1, geom2, min_size=10,
                                  max_size=100)

        geom3 = _ped.Geometry(self._device, 10, 50)
        geom4 = _ped.Geometry(self._device, 30, 40)
        self.c2 = _ped.Constraint(align1, align2, geom3, geom4, min_size=10,
                                  max_size=100)

    def runTest(self):
        startAlign = self.c1.start_align.intersect(self.c2.start_align)
        endAlign = self.c1.end_align.intersect(self.c2.end_align)
        startRange = self.c1.start_range.intersect(self.c2.start_range)
        endRange = self.c1.end_range.intersect(self.c2.end_range)
        minSize = max(self.c1.min_size, self.c2.min_size)
        maxSize = min(self.c1.max_size, self.c2.max_size)

        if not startAlign or not endAlign or not startRange or not endRange:
            expected = None
        else:
            expected = _ped.Constraint(startAlign, endAlign,
                                       startRange, endRange,
                                       min_size=minSize, max_size=maxSize)

        result = self.c1.intersect(self.c2)
        self.assertEqual(result, expected)

class ConstraintSolveMaxTestCase(RequiresDevice):
    def setUp(self):
        RequiresDevice.setUp(self)
        self.c1 = self._device.get_constraint()

    def runTest(self):
        result = self.c1.solve_max()
        self.assertEqual(result.dev, self._device)
        self.assertGreaterEqual(result.length, self._device.length - 1)

class ConstraintSolveNearestTestCase(RequiresDevice):
    def setUp(self):
        RequiresDevice.setUp(self)
        self.c1 = self._device.get_constraint()
        self.g1 = _ped.Geometry(self._device, 1, 8)

    def runTest(self):
        result = self.c1.solve_nearest(self.g1)
        self.assertEqual(result, self.g1)

class ConstraintIsSolutionTestCase(RequiresDevice):
    def setUp(self):
        RequiresDevice.setUp(self)
        self.c1 = self._device.get_constraint()
        self.g1 = _ped.Geometry(self._device, 1, 8)

    def runTest(self):
        self.assertTrue(self.c1.is_solution(self.g1))

class ConstraintStrTestCase(RequiresDevice):
    def setUp(self):
        RequiresDevice.setUp(self)
        align1 = _ped.Alignment(10, 0)
        align2 = _ped.Alignment(10, 0)
        geom1 = _ped.Geometry(self._device, 0, 50)
        geom2 = _ped.Geometry(self._device, 25, 50)

        self.c1 = _ped.Constraint(align1, align2, geom1, geom2, min_size=10,
                                  max_size=100)
    def runTest(self):
        result = str(self.c1).split('\n')

        self.assertEqual(result[0], '_ped.Constraint instance --')
        self.assertTrue(result[1].startswith('  start_align: <_ped.Alignment object at '))
        self.assertNotEqual(result[1].find('  end_align: <_ped.Alignment object at '), -1)
        self.assertTrue(result[2].startswith('  start_range: <_ped.Geometry object at '))
        self.assertNotEqual(result[2].find('  end_range: <_ped.Geometry object at '), -1)
        self.assertEqual(result[3], '  min_size: 10  max_size: 100')



#
# Copyright (C) 2009-2011  Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.
#
# Red Hat Author(s): Chris Lumens <clumens@redhat.com>
#                    David Cantrell <dcantrell@redhat.com>
#

import _ped
import sys
import unittest
from tests.baseclass import RequiresDiskTypes

# One class per method, multiple tests per class.  For these simple methods,
# that seems like good organization.  More complicated methods may require
# multiple classes and their own test suite.
class DiskTypeNewTestCase(unittest.TestCase):
    def runTest(self):
        # You're not allowed to create a new DiskType object by hand.
        self.assertRaises(TypeError, _ped.DiskType)

class DiskTypeGetSetTestCase(RequiresDiskTypes):
    def runTest(self):
        # All attributes are read-only.
        for name in self.disktype.keys():
            t = self.disktype[name]

            self.assertRaises(AttributeError, setattr, t, "name", "fakename")
            self.assertRaises(AttributeError, setattr, t, "features", 47)

            if sys.version_info >= (3,):
                bigint = int
                uni = str
            else:
                bigint = long       # pylint: disable=undefined-variable
                uni = unicode       # pylint: disable=undefined-variable
            self.assertIsInstance(t.name, uni)
            self.assertEqual(t.name, name)
            self.assertIsInstance(t.features, bigint)

class DiskTypeCheckFeatureTestCase(RequiresDiskTypes):
    def runTest(self):
        # The following types have no features [that libparted supports]
        for name in ['aix', 'sun', 'bsd', 'loop']:
            self.assertFalse(self.disktype[name].check_feature(_ped.DISK_TYPE_EXTENDED))
            self.assertFalse(self.disktype[name].check_feature(_ped.DISK_TYPE_PARTITION_NAME))

        # The following types support DISK_TYPE_EXTENDED
        for name in ['msdos']:
            self.assertTrue(self.disktype[name].check_feature(_ped.DISK_TYPE_EXTENDED))
            self.assertFalse(self.disktype[name].check_feature(_ped.DISK_TYPE_PARTITION_NAME))

        # The following types support DISK_TYPE_PARTITION_NAME
        for name in ['amiga', 'gpt', 'mac', 'pc98']:
            self.assertFalse(self.disktype[name].check_feature(_ped.DISK_TYPE_EXTENDED))
            self.assertTrue(self.disktype[name].check_feature(_ped.DISK_TYPE_PARTITION_NAME))

        # The following types support all features
        for name in ['dvh']:
            self.assertTrue(self.disktype[name].check_feature(_ped.DISK_TYPE_EXTENDED))
            self.assertTrue(self.disktype[name].check_feature(_ped.DISK_TYPE_PARTITION_NAME))

class DiskTypeStrTestCase(RequiresDiskTypes):
    def runTest(self):
        self.assertEqual(str(self.disktype['msdos']), '_ped.DiskType instance --\n  name: msdos  features: 1')
        self.assertEqual(str(self.disktype['aix']), '_ped.DiskType instance --\n  name: aix  features: 0')
        self.assertEqual(str(self.disktype['sun']), '_ped.DiskType instance --\n  name: sun  features: 0')
        self.assertEqual(str(self.disktype['amiga']), '_ped.DiskType instance --\n  name: amiga  features: 2')
        self.assertEqual(str(self.disktype['gpt']), '_ped.DiskType instance --\n  name: gpt  features: 2')
        self.assertEqual(str(self.disktype['mac']), '_ped.DiskType instance --\n  name: mac  features: 2')
        self.assertEqual(str(self.disktype['bsd']), '_ped.DiskType instance --\n  name: bsd  features: 0')
        self.assertEqual(str(self.disktype['pc98']), '_ped.DiskType instance --\n  name: pc98  features: 2')
        self.assertEqual(str(self.disktype['loop']), '_ped.DiskType instance --\n  name: loop  features: 0')
        self.assertEqual(str(self.disktype['dvh']), '_ped.DiskType instance --\n  name: dvh  features: 3')



#
# fdisk.py
# Partitioning utility with an fdisk-style interface, but written in
# Python and using the pyparted Python module.
#
# Copyright (C) 2009  Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.
#
# Red Hat Author(s): David Cantrell <dcantrell@redhat.com>
#

import getopt
import os
import parted
import sys

def usage(cmd):
    sys.stdout.write("Usage: %s [-b SSZ] [-u] DISK       Change partition table\n" % (cmd,))
    sys.stdout.write("       %s [-l] [-b SSZ] [-u] DISK  List partition table(s)\n" % (cmd,))
    sys.stdout.write("       %s -s PARTITION             Give partition size(s) in blocks\n\n" % (cmd,))
    sys.stdout.write("Here DISK is something like /dev/hdb or /dev/sda\n")
    sys.stdout.write("and PARTITION is something like /dev/hda7\n\n")
    sys.stdout.write("Options:\n")
    sys.stdout.write("   -b=SSZ, --sectorsize=SSZ    Sector size (in bytes) to use\n")
    sys.stdout.write("   -l, --list                  List partition table(s)\n")
    sys.stdout.write("   -u, --showsectors           Give Start and End in sector units\n")
    sys.stdout.write("   -s, --showblocks            Give Start and End in block units\n")
    sys.stdout.write("   -V, --version               Show fdisk version\n")
    sys.stdout.write("   -?, --help                  Display fdisk usage screen\n")

def displayVersion(cmd):
    ver = parted.version()

    sys.stdout.write("%s:\n" % (cmd,))
    sys.stdout.write("pyparted version: %s.%s.%s\n" % (ver["pyparted"][0], ver["pyparted"][1], ver["pyparted"][2]))
    sys.stdout.write("libparted version: %s\n" % ver["libparted"])

def listPartitionTable(path, sectorsize, showsectors, showblocks):
    device = parted.getDevice(path)
    (cylinders, heads, sectors) = device.biosGeometry
    sizeInBytes = device.length * device.sectorSize
    disk = parted.Disk(device)

    partlist = []
    for partition in disk.partitions:
        if partition.type == parted.PARTITION_PROTECTED or \
           partition.type == parted.PARTITION_METADATA or \
           partition.type == parted.PARTITION_FREESPACE:
            continue

        partlist.append((partition,
                         partition.path,
                         partition.getFlag(parted.PARTITION_BOOT),
                         partition.geometry.start,
                         partition.geometry.end,
                         partition.geometry.length,
                         partition.type,
                         partition.fileSystem))

    colLength = 0
    for parts in partlist:
        path = parts[1]
        if len(path) > colLength:
            colLength = len(path)

    sys.stdout.write("Disk %s: %%d %%s, %d bytes\n" % (disk.device.path, sizeInBytes))
    sys.stdout.write("%d heads, %d sectors/track, %d cylinders\n" % (heads, sectors, cylinders,))
    sys.stdout.write("Units = FIXME\n")
    sys.stdout.write("Disk identifier: FIXME\n\n")

    sys.stdout.write("%-11s %-4s %-11s %-11s %-12s %-4s %s\n" % ("Device", "Boot", "Start", "End", "Blocks", "Id", "System",))

    for parts in partlist:
        (partition, path, bootable, start, end, length, ty, fs) = parts

        if bootable:
            bootflag = '*'
        else:
            bootflag = ''

        sys.stdout.write("%-11s %-4s %-11d %-11d %-12d %-4s" % (path, bootflag, start, end, length, ty,))

        if fs is None:
            # no filesystem, check flags
            if partition.getFlag(parted.PARTITION_SWAP):
                sys.stdout.write(" Linux swap\n")
            elif partition.getFlag(parted.PARTITION_RAID):
                sys.stdout.write(" RAID\n")
            elif partition.getFlag(parted.PARTITION_LVM):
                sys.stdout.write(" Linux LVM\n")
            elif partition.getFlag(parted.PARTITION_HPSERVICE):
                sys.stdout.write(" HP Service\n")
            elif partition.getFlag(parted.PARTITION_PALO):
                sys.stdout.write(" PALO\n")
            elif partition.getFlag(parted.PARTITION_PREP):
                sys.stdout.write(" PReP\n")
            elif partition.getFlag(parted.MSFT_RESERVED):
                sys.stdout.write(" MSFT Reserved\n")
            else:
                sys.stdout.write(" unknown\n")
        else:
            sys.stdout.write(" %s\n" % (fs.type,))

#Disk /dev/sda: 250.0 GB, 250059350016 bytes
#255 heads, 63 sectors/track, 30401 cylinders
#Units = cylinders of 16065 * 512 = 8225280 bytes
#Disk identifier: 0x00000000
#
#   Device Boot      Start         End      Blocks   Id  System
#/dev/sda1               1          26      204819+  ee  GPT
#/dev/sda2              26        4203    33554432   af  Unknown
#/dev/sda3   *        4203        4229      204800   83  Linux
#/dev/sda4            4229       30402   210234515+  8e  Linux LVM

def main(argv):
    cmd = os.path.basename(sys.argv[0])
    opts, args = [], []
    showhelp, showlist, showsectors, showblocks = False, False, False, False

    # These three are unused for now so I'm marking them with an underscore
    # to make pylint happy.
    sectorsize, _cylinders, _heads, _sectors = None, None, None, None

    if len(sys.argv) == 1:
        showhelp = True

    try:
        opts, args = getopt.getopt(sys.argv[1:], "lb:C:H:S:usV?",
                                   ["list", "sectorsize=", "cylinders=",
                                    "heads=", "sectors=", "showsectors",
                                    "showblocks", "version", "help"])
    except getopt.GetoptError:
        showhelp = True

    for o, a in opts:
        if o in ('-l', '--list'):
            showlist = True
        elif o in ('-b', '--sectorsize'):
            sectorsize = a
        elif o in ('-C', '--cylinders'):
            _cylinders = a
        elif o in ('-H', '--heads'):
            _heads = a
        elif o in ('-S', '--sectors'):
            _sectors = a
        elif o in ('-u', '--showsectors'):
            showsectors = True
        elif o in ('-s', '--showblocks'):
            showblocks = True
        elif o in ('-V', '--version'):
            displayVersion(cmd)
            sys.exit(0)
        elif o in ('-?', '--help'):
            usage(cmd)
            sys.exit(0)
        else:
            sys.stderr.write("Invalid option: %s\n\n" % (o,))
            showhelp = True

    if showhelp:
        usage(cmd)
        sys.exit(1)

    for arg in args:
        if showlist:
            listPartitionTable(arg, sectorsize, showsectors, showblocks)

if __name__ == "__main__":
    main(sys.argv)
